package com.piece_framework.piece_ide.form_designer.ui;

import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.piece_framework.piece_ide.form_designer.model.Field;

public class FieldDetailsGeneralSection extends AbstractFieldDetailsSection {
    private Text fNameText;
    private Text fDescriptionText;
    private Button fRequiredYesButton;
    private Button fRequiredNoButton;
    private Text fMessageText;

    @Override
    protected Composite createDetailsComposite(Section parent) {
        parent.setText("tB[hڍ");
        
        GridData layoutData = new GridData();
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.horizontalAlignment = GridData.FILL;
        parent.setLayoutData(layoutData);

        FormToolkit toolkit = getManagedForm().getToolkit();

        Composite composite = toolkit.createComposite(parent);
        composite.setLayout(new GridLayout(2, false));

        layoutData = new GridData();
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.horizontalAlignment = GridData.FILL;
        composite.setLayoutData(layoutData);

        FocusListener focusListener = new FocusListener() {
            public void focusGained(FocusEvent e) {
            }

            public void focusLost(FocusEvent event) {
                if (fNameText == event.widget) {
                    getField().setName(fNameText.getText());
                } else if (fDescriptionText == event.widget) {
                    getField().setDescription(fDescriptionText.getText());
                } else if (fRequiredYesButton == event.widget
                           || fRequiredNoButton == event.widget) {
                    getField().setRequired(fRequiredYesButton.getSelection());
                } else if (fMessageText == event.widget) {
                    getField().setMessage(fMessageText.getText());
                }

                getManagedForm().fireSelectionChanged(
                            getSectionPart(), 
                            new StructuredSelection(getField()));
            }
        };

        fNameText = createLabelAndText(composite, "tB[h");
        fNameText.addFocusListener(focusListener);

        fDescriptionText = createLabelAndText(composite, "Tv");
        fDescriptionText.addFocusListener(focusListener);

        Button[] comboBoxes = createLabelAndRadioButton(composite, "K{");
        fRequiredYesButton = comboBoxes[0];
        fRequiredNoButton = comboBoxes[1];
        fRequiredYesButton.addFocusListener(focusListener);
        fRequiredNoButton.addFocusListener(focusListener);

        fMessageText = createLabelAndText(composite, "bZ[W");
        fMessageText.addFocusListener(focusListener);

        return composite;
    }

    @Override
    protected void selectionFieldChanged(Field field) {
        fNameText.setText("");
        fDescriptionText.setText("");
        fRequiredYesButton.setSelection(false);
        fRequiredNoButton.setSelection(false);
        fMessageText.setText("");

        fNameText.setText(field.getName());
        fDescriptionText.setText(field.getDescription());
        fRequiredYesButton.setSelection(field.isRequired());
        fRequiredNoButton.setSelection(!field.isRequired());
        fMessageText.setText(field.getMessage());
    }
}
