// $Id: FieldsMasterSectionPart.java 28 2008-03-02 18:28:34Z matsu $
package com.piece_framework.piece_ide.form_designer.ui;

import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.IPartSelectionListener;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.piece_framework.piece_ide.form_designer.model.Field;

public class FieldsMasterSectionPart extends SectionPart implements IPartSelectionListener {
    private TableViewer fViewer;
    private Field fCurrentSelection;

    public FieldsMasterSectionPart(Composite parent, IManagedForm managedForm) {
        super(parent, 
              managedForm.getToolkit(), 
              Section.TITLE_BAR | Section.DESCRIPTION);
        initialize(managedForm);

        createContents(getSection(), managedForm.getToolkit());

        getManagedForm().addPart(this);
    }

    private void createContents(Section section, FormToolkit toolkit) {
        section.setText("tB[hꗗ");
        section.setDescription("ҏWtB[hIĂB");

        Composite composite = toolkit.createComposite(section);
        composite.setLayout(new GridLayout(2, false));

        createTable(composite);
        createButtons(composite);

        section.setClient(composite);
    }

    private void createTable(Composite composite) {
        Table table = getManagedForm().getToolkit().createTable(
                        composite, 
                        SWT.SINGLE | SWT.FULL_SELECTION);
        table.setHeaderVisible(true);
        table.setLinesVisible(true);

        GridData layoutData = new GridData();
        layoutData.horizontalAlignment = GridData.FILL;
        layoutData.verticalAlignment = GridData.FILL;
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.grabExcessVerticalSpace = true;
        table.setLayoutData(layoutData);
       
        createTableColumn(table, "tB[h", 100);
        createTableColumn(table, "", 100);

        fViewer = new TableViewer(table);
        fViewer.setContentProvider(new ArrayContentProvider());
        fViewer.setLabelProvider(new ITableLabelProvider() {
            public Image getColumnImage(Object element, int columnIndex) {
                return null;
            }

            public String getColumnText(Object element, int columnIndex) {
                if (element == null) {
                    return "";
                }
                if (!(element instanceof Field)) {
                    return "";
                }
                
                Field field = (Field) element;
                if (columnIndex == 0) {
                    return field.getName();
                } else if (columnIndex == 1) {
                    return field.getDescription();
                }
                return "";
            }

            public void addListener(ILabelProviderListener listener) {
            }

            public void dispose() {
            }

            public boolean isLabelProperty(Object element, String property) {
                return false;
            }

            public void removeListener(ILabelProviderListener listener) {
            }
        });

        fViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                Field field = (Field) ((IStructuredSelection)
                        event.getSelection()).getFirstElement();
                if (fCurrentSelection == field) {
                    return;
                }
                fCurrentSelection = field;
                
                FieldsMasterSectionPart.this.getManagedForm()
                    .fireSelectionChanged(
                                FieldsMasterSectionPart.this,  
                                event.getSelection());
            }
        });
    }


    private void createButtons(Composite composite) {
        final Composite buttons = 
            getManagedForm().getToolkit().createComposite(composite);
        GridData layoutData = new GridData();
        layoutData.verticalAlignment = GridData.FILL;
        buttons.setLayoutData(layoutData);

        buttons.setLayout(new GridLayout());

        Button addButton = createButton(buttons, "ǉ(&A)...");
        addButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                InputDialog dialog = new InputDialog(
                                        buttons.getShell(), 
                                        "tB[h", 
                                        "tB[h͂ĂB", 
                                        null, 
                                        null);
                dialog.open();

                Field field = new Field(dialog.getValue());
                fViewer.add(field);
                fViewer.setSelection(new StructuredSelection(field));
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button delButton = createButton(buttons, "폜(&D)...");
        delButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                Field field = 
                    (Field) ((IStructuredSelection) fViewer.getSelection())
                    .getFirstElement();
                fViewer.remove(field);
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button upButton = createButton(buttons, "");
        upButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                int index = fViewer.getTable().getSelectionIndex();
                if (index == 0) {
                    return;
                }

                Field tmpField = (Field) fViewer.getElementAt(index - 1);
                fViewer.replace(fViewer.getElementAt(index), index - 1);
                fViewer.replace(tmpField, index);

                fViewer.setSelection(
                        new StructuredSelection(
                                (Field) fViewer.getElementAt(index - 1)));
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button downButton = createButton(buttons, "");
        downButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                int index = fViewer.getTable().getSelectionIndex();
                if (index + 1 == fViewer.getTable().getItemCount()) {
                    return;
                }

                Field tmpField = (Field) fViewer.getElementAt(index + 1);
                fViewer.replace(fViewer.getElementAt(index), index + 1);
                fViewer.replace(tmpField, index);

                fViewer.setSelection(
                        new StructuredSelection(
                                (Field) fViewer.getElementAt(index + 1)));
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });
    }

    private TableColumn createTableColumn(
                            final Table table, 
                            final String text, 
                            final int width) {
        TableColumn column = new TableColumn(table, SWT.NULL);
        column.setText(text);
        column.setWidth(width);

        return column;
    }

    private Button createButton(final Composite parent, final String label) {
        Button button = getManagedForm().getToolkit().createButton(
                                parent, 
                                label, 
                                SWT.PUSH);

        GridData layoutData = new GridData();
        layoutData.horizontalAlignment = GridData.FILL;
        button.setLayoutData(layoutData);

        return button;
    }

    public void selectionChanged(IFormPart part, ISelection selection) {
        if (part == this) {
            return;
        }
        if (!(selection instanceof IStructuredSelection)) {
            return;
        }

        fViewer.refresh(
                (Field) ((IStructuredSelection) selection).getFirstElement());
    }
}
